"use strict";
var graphql_1 = require("graphql");
var graphql_server_core_1 = require("graphql-server-core");
var rabbitmq_pub_sub_1 = require("rabbitmq-pub-sub");
var childLogger_1 = require("./childLogger");
var AMQPSubscription = (function () {
    function AMQPSubscription(options) {
        this.GRAPHQL_QUEUENAME = "graphql";
        if (!options) {
            throw new Error('Apollo Server requires options.');
        }
        if (arguments.length > 1) {
            throw new Error("Apollo Server expects exactly one argument, got " + arguments.length);
        }
        var config = options.config || { host: "127.0.0.1", port: 5672 };
        var logger = options.logger;
        this.logger = childLogger_1.createChildLogger(logger, 'AmqpSubscriptionServer');
        var factory = new rabbitmq_pub_sub_1.RabbitMqConnectionFactory(logger, config);
        this.listener = new rabbitmq_pub_sub_1.RabbitMqConsumer(logger, factory);
        this.graphqlOptions = options;
    }
    AMQPSubscription.prototype.listenToQueries = function (cb) {
        var _this = this;
        return new Promise(function (resolve, reject) { return _this.listener.subscribe(_this.GRAPHQL_QUEUENAME, function (msg) { return _this.onMessage(msg).then(function (m) { return cb ? cb(m) : m; }); })
            .then(function (disposer) {
            _this.unsubscribeChannel = disposer;
            return resolve();
        }).catch(function (err) {
            _this.logger.error(err, "failed to recieve message from queue '%s'", _this.GRAPHQL_QUEUENAME);
            reject();
        }); });
    };
    AMQPSubscription.prototype.unsubscribe = function () {
        var _this = this;
        this.unsubscribeChannel().then(function () {
            _this.logger.trace("cancelled channel from subscribing to queue '%s'", _this.GRAPHQL_QUEUENAME);
        }).catch(function (err) {
            _this.logger.error(err, "channel cancellation failed from queue '%j'", _this.GRAPHQL_QUEUENAME);
        });
    };
    AMQPSubscription.prototype.onMessage = function (queryOptions) {
        this.logger.trace("message received to process is '(%j)'", queryOptions);
        var isBatch = false;
        var context = this.graphqlOptions.context;
        if (isBatch) {
            context = Object.assign({}, context || {});
        }
        var formatErrorFn = this.graphqlOptions.formatError || graphql_1.formatError;
        var params = {
            schema: this.graphqlOptions.schema,
            query: queryOptions.query,
            variables: queryOptions.variables,
            context: context,
            rootValue: this.graphqlOptions.rootValue,
            operationName: queryOptions.operationName,
            logFunction: this.graphqlOptions.logFunction,
            validationRules: this.graphqlOptions.validationRules,
            formatError: formatErrorFn,
            formatResponse: this.graphqlOptions.formatResponse,
            debug: this.graphqlOptions.debug,
        };
        if (this.graphqlOptions.formatParams) {
            params = this.graphqlOptions.formatParams(params);
        }
        return graphql_server_core_1.runQuery(params);
    };
    return AMQPSubscription;
}());
exports.AMQPSubscription = AMQPSubscription;
//# sourceMappingURL=rabbitmqApollo.js.map